#!/usr/bin/env python
# -*- coding: utf-8 -*-

# # Analyse de survie

# In[1]:


from jyquickhelper import add_notebook_menu
add_notebook_menu()


# ## Quelques données
# 
# On récupère les données disponibles sur *open.data.gouv.fr* [Données hospitalières relatives à l'épidémie de COVID-19](https://www.data.gouv.fr/fr/datasets/donnees-hospitalieres-relatives-a-lepidemie-de-covid-19/). Ces données ne permettent pas de construire la courbe de [Kaplan-Meier](https://fr.wikipedia.org/wiki/Estimateur_de_Kaplan-Meier). On sait combien de personnes rentrent et sortent chaque jour mais on ne sait pas quand une personne qui sort un 1er avril est entrée.

# In[2]:


import numpy.random as rnd

import pandas
df = pandas.read_csv("https://www.data.gouv.fr/en/datasets/r/63352e38-d353-4b54-bfd1-f1b3ee1cabd7", sep=";")
gr = df[["jour", "rad", "dc"]].groupby(["jour"]).sum()
diff = gr.diff().reset_index(drop=False)
diff.head()


# In[3]:


def donnees_artificielles(hosp, mu=14, nu=21):
    dt = pandas.to_datetime(hosp['jour'])
    res = []
    for i in range(hosp.shape[0]):
        date = dt[i].dayofyear
        h = hosp.iloc[i, 1]
        delay = rnd.exponential(mu, int(h))
        for j in range(delay.shape[0]):
            res.append([date - int(delay[j]), date, 1])
        h = hosp.iloc[i, 2]
        delay = rnd.exponential(nu, int(h))
        for j in range(delay.shape[0]):
            res.append([date - int(delay[j]), date , 0])
    return pandas.DataFrame(res, columns=["entree", "sortie", "issue"])


data = donnees_artificielles(diff[1:].reset_index(drop=True)).sort_values('entree')
data.head()


# Chaque ligne est une personne, `entree` est le jour d'entrée à l'hôpital, `sortie` celui de la sortie, `issue`, 0 pour décès, 1 pour en vie.

# In[4]:


data.describe()


# Il y a environ 80% de survie dans ces données.

# In[5]:


import numpy
duree = data.sortie - data.entree
deces = (data.issue == 0).astype(numpy.int32)


# In[6]:


import numpy
import matplotlib.pyplot as plt
from lifelines import KaplanMeierFitter
fig, ax = plt.subplots(1, 1, figsize=(10, 4))
kmf = KaplanMeierFitter()
kmf.fit(duree, deces)
kmf.plot(ax=ax)
ax.legend();


# ## Régression de Cox
# 
# On reprend les données artificiellement générées et on ajoute une variable identique à la durée plus un bruit mais quasi nul 

# In[7]:


import pandas
data_simple = pandas.DataFrame({'duree': duree, 'deces': deces,
                                'X1': duree * 0.57 * deces + numpy.random.randn(duree.shape[0]),
                                'X2': duree * (-0.57) * deces + numpy.random.randn(duree.shape[0])})
data_simple.head()


# In[8]:


from sklearn.model_selection import train_test_split
data_train, data_test = train_test_split(data_simple, test_size=0.8)


# In[9]:


from lifelines.fitters.coxph_fitter import CoxPHFitter
cox = CoxPHFitter()
cox.fit(data_train[['duree', 'deces', 'X1']], duration_col="duree", event_col="deces",
        show_progress=True)


# In[10]:


cox.print_summary()


# In[11]:


cox2 = CoxPHFitter()
cox2.fit(data_train[['duree', 'deces', 'X2']], duration_col="duree", event_col="deces",
        show_progress=True)
cox2.print_summary()


# In[12]:


cox.predict_cumulative_hazard(data_test[:5])


# In[13]:


cox.predict_survival_function(data_test[:5])


# In[14]:





# In[15]: